@@ -114,7 +114,7 @@ static int (CDECL *p__ltoa_s)(LONG, char*, size_t, int);
 static char* (CDECL *p__get_narrow_winmain_command_line)(void);
 static int (CDECL *p_sopen_dispatch)(const char *, int, int, int, int *, int);
 static int (CDECL *p_sopen_s)(int *, const char *, int, int, int);
-static MSVCRT_lldiv_t (CDECL *p_lldiv)(LONGLONG,LONGLONG);
+static MSVCRT_lldiv_t* (CDECL *p_lldiv)(MSVCRT_lldiv_t*,LONGLONG,LONGLONG);
 static int (CDECL *p__isctype)(int,int);
 static int (CDECL *p_isblank)(int);
 static int (CDECL *p__isblank_l)(int,_locale_t);
@@ -125,6 +125,10 @@ static int (CDECL *p_fesetround)(int);
 static void (CDECL *p___setusermatherr)(MSVCRT_matherr_func);
 static int* (CDECL *p_errno)(void);
 static char* (CDECL *p_asctime)(const struct tm *);
+static void (CDECL *p_exit)(int);
+static int (CDECL *p__crt_atexit)(void (CDECL*)(void));
+static int (__cdecl *p_crt_at_quick_exit)(void (__cdecl *func)(void));
+static void (__cdecl *p_quick_exit)(int exitcode);
 
 static void test__initialize_onexit_table(void)
 {
@@ -429,6 +433,10 @@ static BOOL init(void)
     p___setusermatherr = (void*)GetProcAddress(module, "__setusermatherr");
     p_errno = (void*)GetProcAddress(module, "_errno");
     p_asctime = (void*)GetProcAddress(module, "asctime");
+    p__crt_atexit = (void*)GetProcAddress(module, "_crt_atexit");
+    p_exit = (void*)GetProcAddress(module, "exit");
+    p_crt_at_quick_exit = (void*)GetProcAddress(module, "_crt_at_quick_exit");
+    p_quick_exit = (void*)GetProcAddress(module, "quick_exit");
 
     return TRUE;
 }
@@ -504,7 +512,7 @@ static void test_lldiv(void)
 {
     MSVCRT_lldiv_t r;
 
-    r = p_lldiv((LONGLONG)0x111 << 32 | 0x222, (LONGLONG)1 << 32);
+    p_lldiv(&r, (LONGLONG)0x111 << 32 | 0x222, (LONGLONG)1 << 32);
     ok(r.quot == 0x111, "quot = %s\n", wine_dbgstr_longlong(r.quot));
     ok(r.rem == 0x222, "rem = %s\n", wine_dbgstr_longlong(r.rem));
 }
@@ -703,7 +711,7 @@ static void test_math_errors(void)
     /* necessary so that exp(1e100)==INFINITY on glibc, we can remove this if we change our implementation */
     p_fesetround(FE_TONEAREST);
 
-    for(i = 0; i < sizeof(testsd)/sizeof(testsd[0]); i++) {
+    for(i = 0; i < ARRAY_SIZE(testsd); i++) {
         p_funcd = (void*)GetProcAddress(module, testsd[i].func);
         *p_errno() = -1;
         exception.type = -1;
@@ -717,7 +725,7 @@ static void test_math_errors(void)
            "%s(%f) got exception arg1 %f\n", testsd[i].func, testsd[i].x, exception.arg1);
     }
 
-    for(i = 0; i < sizeof(tests2d)/sizeof(tests2d[0]); i++) {
+    for(i = 0; i < ARRAY_SIZE(tests2d); i++) {
         p_func2d = (void*)GetProcAddress(module, tests2d[i].func);
         *p_errno() = -1;
         exception.type = -1;
@@ -733,7 +741,7 @@ static void test_math_errors(void)
            "%s(%f, %f) got exception arg2 %f\n", tests2d[i].func, tests2d[i].a, tests2d[i].b, exception.arg2);
     }
 
-    for(i = 0; i < sizeof(testsdl)/sizeof(testsdl[0]); i++) {
+    for(i = 0; i < ARRAY_SIZE(testsdl); i++) {
         p_funcdl = (void*)GetProcAddress(module, testsdl[i].func);
         *p_errno() = -1;
         exception.type = -1;
@@ -765,6 +773,164 @@ static void test_asctime(void)
     ok(!strcmp(ret, "Thu Jan  1 00:00:00 1970\n"), "asctime returned %s\n", ret);
 }
 
+static LONG* get_failures_counter(HANDLE *map)
+{
+    *map = CreateFileMappingA(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE,
+            0, sizeof(LONG), "winetest_failures_counter");
+    return MapViewOfFile(*map, FILE_MAP_ALL_ACCESS, 0, 0, sizeof(LONG));
+}
+
+static void free_failures_counter(LONG *mem, HANDLE map)
+{
+    UnmapViewOfFile(mem);
+    CloseHandle(map);
+}
+
+static void set_failures_counter(LONG add)
+{
+    HANDLE failures_map;
+    LONG *failures;
+
+    failures = get_failures_counter(&failures_map);
+    *failures = add;
+    free_failures_counter(failures, failures_map);
+}
+
+static void test_exit(const char *argv0)
+{
+    PROCESS_INFORMATION proc;
+    STARTUPINFOA startup = {0};
+    char path[MAX_PATH];
+    HANDLE failures_map, exit_event, quick_exit_event;
+    LONG *failures;
+    DWORD ret;
+
+    exit_event = CreateEventA(NULL, FALSE, FALSE, "exit_event");
+    quick_exit_event = CreateEventA(NULL, FALSE, FALSE, "quick_exit_event");
+
+    failures = get_failures_counter(&failures_map);
+    sprintf(path, "%s misc exit", argv0);
+    startup.cb = sizeof(startup);
+    CreateProcessA(NULL, path, NULL, NULL, TRUE, 0, NULL, NULL, &startup, &proc);
+    ret = WaitForSingleObject(proc.hProcess, INFINITE);
+    ok(ret == WAIT_OBJECT_0, "child process wait failed\n");
+    GetExitCodeProcess(proc.hProcess, &ret);
+    ok(ret == 1, "child process exited with code %d\n", ret);
+    CloseHandle(proc.hProcess);
+    CloseHandle(proc.hThread);
+    ok(!*failures, "%d tests failed in child process\n", *failures);
+    free_failures_counter(failures, failures_map);
+
+
+    ret = WaitForSingleObject(exit_event, 0);
+    ok(ret == WAIT_OBJECT_0, "exit_event was not set (%x)\n", ret);
+    ret = WaitForSingleObject(quick_exit_event, 0);
+    ok(ret == WAIT_TIMEOUT, "quick_exit_event should not have be set (%x)\n", ret);
+
+    CloseHandle(exit_event);
+    CloseHandle(quick_exit_event);
+}
+
+static int atexit_called;
+
+static void CDECL at_exit_func1(void)
+{
+    HANDLE exit_event = CreateEventA(NULL, FALSE, FALSE, "exit_event");
+
+    ok(exit_event != NULL, "CreateEvent failed: %d\n", GetLastError());
+    ok(atexit_called == 1, "atexit_called = %d\n", atexit_called);
+    atexit_called++;
+    SetEvent(exit_event);
+    CloseHandle(exit_event);
+    set_failures_counter(winetest_get_failures());
+}
+
+static void CDECL at_exit_func2(void)
+{
+    ok(!atexit_called, "atexit_called = %d\n", atexit_called);
+    atexit_called++;
+    set_failures_counter(winetest_get_failures());
+}
+
+static int atquick_exit_called;
+
+static void CDECL at_quick_exit_func1(void)
+{
+    HANDLE quick_exit_event = CreateEventA(NULL, FALSE, FALSE, "quick_exit_event");
+
+    ok(quick_exit_event != NULL, "CreateEvent failed: %d\n", GetLastError());
+    ok(atquick_exit_called == 1, "atquick_exit_called = %d\n", atquick_exit_called);
+    atquick_exit_called++;
+    SetEvent(quick_exit_event);
+    CloseHandle(quick_exit_event);
+    set_failures_counter(winetest_get_failures());
+}
+
+static void CDECL at_quick_exit_func2(void)
+{
+    ok(!atquick_exit_called, "atquick_exit_called = %d\n", atquick_exit_called);
+    atquick_exit_called++;
+    set_failures_counter(winetest_get_failures());
+}
+
+static void test_call_exit(void)
+{
+    ok(!p__crt_atexit(at_exit_func1), "_crt_atexit failed\n");
+    ok(!p__crt_atexit(at_exit_func2), "_crt_atexit failed\n");
+
+    ok(!p_crt_at_quick_exit(at_quick_exit_func1), "_crt_at_quick_exit failed\n");
+    ok(!p_crt_at_quick_exit(at_quick_exit_func2), "_crt_at_quick_exit failed\n");
+
+    set_failures_counter(winetest_get_failures());
+    p_exit(1);
+}
+
+static void test_call_quick_exit(void)
+{
+    ok(!p__crt_atexit(at_exit_func1), "_crt_atexit failed\n");
+    ok(!p__crt_atexit(at_exit_func2), "_crt_atexit failed\n");
+
+    ok(!p_crt_at_quick_exit(at_quick_exit_func1), "_crt_at_quick_exit failed\n");
+    ok(!p_crt_at_quick_exit(at_quick_exit_func2), "_crt_at_quick_exit failed\n");
+
+    set_failures_counter(winetest_get_failures());
+    p_quick_exit(2);
+}
+
+static void test_quick_exit(const char *argv0)
+{
+    PROCESS_INFORMATION proc;
+    STARTUPINFOA startup = {0};
+    char path[MAX_PATH];
+    HANDLE failures_map, exit_event, quick_exit_event;
+    LONG *failures;
+    DWORD ret;
+
+    exit_event = CreateEventA(NULL, FALSE, FALSE, "exit_event");
+    quick_exit_event = CreateEventA(NULL, FALSE, FALSE, "quick_exit_event");
+
+    failures = get_failures_counter(&failures_map);
+    sprintf(path, "%s misc quick_exit", argv0);
+    startup.cb = sizeof(startup);
+    CreateProcessA(NULL, path, NULL, NULL, TRUE, 0, NULL, NULL, &startup, &proc);
+    ret = WaitForSingleObject(proc.hProcess, INFINITE);
+    ok(ret == WAIT_OBJECT_0, "child process wait failed\n");
+    GetExitCodeProcess(proc.hProcess, &ret);
+    ok(ret == 2, "child process exited with code %d\n", ret);
+    CloseHandle(proc.hProcess);
+    CloseHandle(proc.hThread);
+    ok(!*failures, "%d tests failed in child process\n", *failures);
+    free_failures_counter(failures, failures_map);
+
+    ret = WaitForSingleObject(quick_exit_event, 0);
+    ok(ret == WAIT_OBJECT_0, "quick_exit_event was not set (%x)\n", ret);
+    ret = WaitForSingleObject(exit_event, 0);
+    ok(ret == WAIT_TIMEOUT, "exit_event should not have be set (%x)\n", ret);
+
+    CloseHandle(exit_event);
+    CloseHandle(quick_exit_event);
+}
+
 START_TEST(misc)
 {
     int arg_c;
@@ -775,7 +941,12 @@ START_TEST(misc)
 
     arg_c = winetest_get_mainargs(&arg_v);
     if(arg_c == 3) {
-        test__get_narrow_winmain_command_line(NULL);
+        if(!strcmp(arg_v[2], "cmd"))
+            test__get_narrow_winmain_command_line(NULL);
+        else if(!strcmp(arg_v[2], "exit"))
+            test_call_exit();
+        else if(!strcmp(arg_v[2], "quick_exit"))
+            test_call_quick_exit();
         return;
     }
 
@@ -791,4 +962,6 @@ START_TEST(misc)
     test_isblank();
     test_math_errors();
     test_asctime();
+    test_exit(arg_v[0]);
+    test_quick_exit(arg_v[0]);
 }
